<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***/
class MATCHMAKING_CMP_Notification extends OW_Component
{
    private $items = array();
    private $user;
    private $lastUserId=0;

    const NL_PLACEHOLDER = '%%%nl%%%';
    const TAB_PLACEHOLDER = '%%%tab%%%';
    const SPACE_PLACEHOLDER = '%%%space%%%';

    public function __construct( $user, $items )
    {
        parent::__construct();

        $this->user = $user;
        $this->items = $items;
    }

    public function onBeforeRender()
    {
        parent::onBeforeRender();

        $userService = BOL_UserService::getInstance();
        $matchService = MATCHMAKING_BOL_Service::getInstance();

        $idList = array();
        $compatibilityList = array();
        foreach ( $this->items as $item )
        {
            $idList[] = $item['id'];
            $compatibilityList[$item['id']] = $matchService->getCompatibilityByValue($item['compatibility']);
            if ($item['id'] > $this->lastUserId)
            {
                $this->lastUserId = $item['id'];
            }
        }

        $avatars = array();
        $usernameList = array();
        $displayNameList = array();
        $onlineInfo = array();

        $list = array();
        if ( !empty($idList) )
        {
            $avatars = BOL_AvatarService::getInstance()->getAvatarsUrlList($idList, 2);

            foreach ($idList as $userId)
            {
                $list[$userId]['userUrl'] = $userService->getUserUrl($userId);
                $list[$userId]['displayName'] = $userService->getDisplayName($userId);
                $list[$userId]['avatarUrl'] = $avatars[$userId];
                $list[$userId]['compatibility'] = $compatibilityList[$userId];

                $fields = $matchService->getFieldsForEmail($userId);
                $list[$userId]['sex'] = empty($fields['sex']) ? '' : $fields['sex'];
                $list[$userId]['age'] = empty($fields['age']) ? '' : $fields['age'];
                $list[$userId]['googlemap_location'] = empty($fields['googlemap_location']) ? '' : $fields['googlemap_location'];
            }
        }

        $this->assign('userName', BOL_UserService::getInstance()->getDisplayName($this->user->getId()));
        $this->assign('list', $list);
        $this->assign('matchesUrl', OW::getRouter()->urlForRoute('matchmaking_members_page'));
    }

    private function getSubject()
    {
        return OW::getLanguage()->text('matchmaking', 'email_notifications_send_new_matches_subject');
    }

    private function getHtml()
    {
        $template = OW::getPluginManager()->getPlugin('matchmaking')->getCmpViewDir() . 'notification_html.html';
        $this->setTemplate($template);

        return parent::render();
    }

    private function getTxt()
    {
        $template = OW::getPluginManager()->getPlugin('matchmaking')->getCmpViewDir() . 'notification_txt.html';
        $this->setTemplate($template);

        $this->assign('nl', self::NL_PLACEHOLDER);
        $this->assign('tab', self::TAB_PLACEHOLDER);
        $this->assign('space', self::SPACE_PLACEHOLDER);

        $content = parent::render();
        $search = array(self::NL_PLACEHOLDER, self::TAB_PLACEHOLDER, self::SPACE_PLACEHOLDER);
        $replace = array("\n", '    ', ' ');

        return str_replace($search, $replace, $content);
    }

    public function sendNotification()
    {
        $subject = $this->getSubject();
        $txt = $this->getTxt();
        $html = $this->getHtml();

        $mail = OW::getMailer()->createMail()
            ->addRecipientEmail($this->user->email)
            ->setTextContent($txt)
            ->setHtmlContent($html)
            ->setSubject($subject);

        OW::getMailer()->send($mail);

//        BOL_PreferenceService::getInstance()->savePreferenceValue('matchmaking_lastmatch_userid', (int)$this->items[0]['id'], $this->user->getId());
        BOL_PreferenceService::getInstance()->savePreferenceValue('matchmaking_lastmatch_userid', (int)$this->lastUserId, $this->user->getId());
    }
}