<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***/
class MATCHMAKING_Cron extends OW_Cron
{
    const CRON_USER_COUNT = 100;

    /**
     *
     * @var MATCHMAKING_BOL_Service
     */
    private $service;
    private $send_new_matches_interval;
    private $last_matches_sent_timestamp;

    public function __construct()
    {
        parent::__construct();

        $this->service = MATCHMAKING_BOL_Service::getInstance();

        $this->send_new_matches_interval = 86400 * (int)OW::getConfig()->getValue('matchmaking', 'send_new_matches_interval');
        $this->last_matches_sent_timestamp = OW::getConfig()->getValue('matchmaking', 'last_matches_sent_timestamp');
        $this->addJob('sendNewMatches', 1);
    }

    public function run()
    {

    }

    /**
     * Send new matches to users by email
     */
    public function sendNewMatches()
    {
        $config = OW::getConfig();

        if ( !$config->configExists('matchmaking', 'cron_busy') )
        {
            $config->addConfig('matchmaking', 'cron_busy', 0, 'Mass mailing queue is busy');
        }

        if ( !$config->configExists('matchmaking', 'cron_mailing_user_first') )
        {
            $config->addConfig('matchmaking', 'cron_mailing_user_first', 0, 'Already mailed users count');
        }

        // check if cron queue is not busy
        if ( $config->getValue('matchmaking', 'cron_busy') )
        {
            $cronBusyTime = (int)$config->getValue('matchmaking', 'cron_busy_timestamp');
            if (time()-$cronBusyTime > 60 * 30)
            {
                $config->saveConfig('matchmaking', 'cron_busy', 0);
            }
            return;
        }

        if ($this->send_new_matches_interval == 0)
        {
            return;
        }

        if ( time()-$this->last_matches_sent_timestamp < $this->send_new_matches_interval )
        {
            return;
        }

        $config->saveConfig('matchmaking', 'cron_busy', 1);

        if ( !$config->configExists('matchmaking', 'cron_busy_timestamp') )
        {
            $config->addConfig('matchmaking', 'cron_busy_timestamp', time(), '');
        }
        else
        {
            $config->saveConfig('matchmaking', 'cron_busy_timestamp', time());
        }

        $userService = BOL_UserService::getInstance();

        $first = $config->getValue('matchmaking', 'cron_mailing_user_first');

        $count = $userService->count(true);

        if ( $first < $count )
        {
            $users = $userService->findList($first, self::CRON_USER_COUNT, true);
            $counter = 0;
            /**
             * @var BOL_User $user
             */
            foreach ( $users as $id => $user )
            {
                if ($user->emailVerify == 0)
                {
                    $counter++;
                    continue;
                }

                try
                {
                    $this->service->sendNewMatchesForUser($user->getId());
                    $counter++;
                }
                catch ( Exception $e )
                {
                    $config->saveConfig('matchmaking', 'cron_mailing_user_first', $first + $counter);
                    $config->saveConfig('matchmaking', 'cron_busy', 0);
                    return;
                }
            }

            $config->saveConfig('matchmaking', 'cron_mailing_user_first', $first + $counter);
        }
        else
        {
            OW::getConfig()->saveConfig('matchmaking', 'cron_mailing_user_first', 0);
            OW::getConfig()->saveConfig('matchmaking', 'last_matches_sent_timestamp', time());
        }

        $config->saveConfig('matchmaking', 'cron_busy', 0);
    }
}