<?php

/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/

final class OXDATE_BOL_AccountTypeToGenderService
{
    /**
     * Singleton instance.
     *
     * @var OXDATE_BOL_AccountTypeToGenderService
     */
    private static $classInstance;
    private $accountTypeToGenderDao;
    private $cache;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return OXDATE_BOL_AccountTypeToGenderService
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * Constructor.
     */
    private function __construct()
    {
        $this->accountTypeToGenderDao = OXDATE_BOL_AccountTypeToGenderDao::getInstance();
        $this->cache = array();
    }

    public function updateGenderValues()
    {
        $this->cache = array();

        $genderToAccountTypeList = $this->accountTypeToGenderDao->findAll();
        $accountTypesList = BOL_QuestionService::getInstance()->findAllAccountTypes();

        $genderValues = array();
        $accountTypes = array();
        $accountNameList = array();
        $questionValues = array();

        $deleteList = array();
        $addList = array();

        foreach ( $accountTypesList as $value )
        {
            $accountTypes[$value->name] = $value;
            $accountNameList[$value->name] = $value->name;
        }

        foreach ( $genderToAccountTypeList as $value )
        {
            if ( !empty($accountTypes[$value->accountType]) )
            {
                $genderValues[$value->genderValue] = $value->accountType;
            }
            else
            {
                $deleteList[$value->genderValue] = $value->genderValue;
            }
        }

        $addList = array_diff($accountNameList, $genderValues);

        // delete not exists gender values
        if ( !empty($deleteList) )
        {
            BOL_QuestionService::getInstance()->deleteQuestionValues('sex', $deleteList);
            $this->deleteByGenderList($deleteList);
        }

        // get question values dto
        $values = BOL_QuestionService::getInstance()->findQuestionValues('sex');

        foreach ( $values as $value )
        {
            $questionValues[$value->value] = $value;
        }

        // update gender values
        $itemNumber = 0;

        for ( $i = 0; $i < 31; $i++ )
        {
            $intValue = pow(2, $i);

            if ( empty($questionValues[$intValue]) )
            {
                $accountType = array_shift($addList);

                // question has empty value
                if ( !empty($accountType) )
                {
                    // add values
                    $value = new BOL_QuestionValue();
                    $value->questionName = 'sex';
                    $value->value = $intValue;
                    $value->sortOrder = $accountTypes[$accountType]->sortOrder;

                    BOL_QuestionService::getInstance()->saveOrUpdateQuestionValue($value);

                    $a2gDto = new OXDATE_BOL_AccountTypeToGender();
                    $a2gDto->accountType = $accountType;
                    $a2gDto->genderValue = $intValue;

                    $this->save($a2gDto);

                    $itemNumber++;
                }
            }
            else if ( empty($genderValues[$intValue]) )
            {
                $accountType = array_shift($addList);

                // question has value which not join to account type
                if ( !empty($accountType) )
                {
                    $a2gDto = new OXDATE_BOL_AccountTypeToGender();
                    $a2gDto->accountType = $accountType;
                    $a2gDto->genderValue = $intValue;

                    $this->save($a2gDto);

                    $itemNumber++;
                }
                else
                {
                    BOL_QuestionService::getInstance()->deleteQuestionValue('sex', $intValue);
                }
            }
            else
            {
                // question has value which join to account type
                $accountType = $genderValues[$intValue];

                if ( $questionValues[$intValue]->sortOrder != $accountTypes[$accountType]->sortOrder )
                {
                    // update values order
                    $value = $questionValues[$intValue];
                    $value->sortOrder = $accountTypes[$accountType]->sortOrder;

                    BOL_QuestionService::getInstance()->saveOrUpdateQuestionValue($value);
                }
            }
        }
    }

    public function findAll()
    {
        if ( empty($this->cache) )
        {
            $this->cache = $this->accountTypeToGenderDao->findAll();
        }

        return $this->cache;
    }

    public function getGender( $accountType )
    {
        if ( empty($accountType) )
        {
            return null;
        }

        $dtoList = $this->findAll();

        /* @var $dto OXDATE_BOL_AccountTypeToGender */
        foreach ( $dtoList as $dto )
        {
            if ( $accountType == $dto->accountType )
            {
                return $dto->genderValue;
            }
        }

        return null;
    }

    public function getAccountType( $gender )
    {
        if ( empty($gender) )
        {
            return null;
        }

        $dtoList = $this->findAll();

        /* @var $dto OXDATE_BOL_AccountTypeToGender */
        foreach ( $dtoList as $dto )
        {
            if ( $gender == $dto->genderValue )
            {
                return $dto->accountType;
            }
        }

        return null;
    }

    public function deleteByAccountTypeList( array $accountTypeList )
    {
        $this->cache = array();
        $this->accountTypeToGenderDao->deleteByAccountTypeList($accountTypeList);
    }

    public function deleteByGenderList( array $genderlist )
    {
        $this->cache = array();
        $this->accountTypeToGenderDao->deleteByGenderList($genderlist);
    }

    public function save( OXDATE_BOL_AccountTypeToGender $dto )
    {
        $this->accountTypeToGenderDao->batchReplace(array($dto));
    }
}
