<?php

/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class USERCREDITS_BOL_LogDao extends OW_BaseDao
{
    /**
     * Singleton instance.
     *
     * @var USERCREDITS_BOL_LogDao
     */
    private static $classInstance;

    /**
     * Constructor.
     *
     */
    protected function __construct()
    {
        parent::__construct();
    }

    /**
     * Returns an instance of class.
     *
     * @return USERCREDITS_BOL_LogDao
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * @see OW_BaseDao::getDtoClassName()
     *
     */
    public function getDtoClassName()
    {
        return 'USERCREDITS_BOL_Log';
    }

    /**
     * @see OW_BaseDao::getTableName()
     *
     */
    public function getTableName()
    {
        return OW_DB_PREFIX . 'usercredits_log';
    }
    
    /**
     * Finds user last action log
     * 
     * @param int $userId
     * @param int $actionId
     * @return USERCREDITS_BOL_Log
     */
    public function findLast( $userId, $actionId )
    {
        $example = new OW_Example();
        $example->andFieldEqual('userId', $userId);
        $example->andFieldEqual('actionId', $actionId);
        $example->setOrder('`logTimestamp` DESC');
        $example->setLimitClause(0, 1);
        
        return $this->findObjectByExample($example);
    }

    /**
     * @param $userId
     * @param $page
     * @param $limit
     * @return array
     */
    public function findListForUser( $userId, $page, $limit )
    {
        $actionDao = USERCREDITS_BOL_ActionDao::getInstance();

        $start = ($page - 1) * $limit;
        $sql =
            'SELECT l.id,l.userId,l.actionId,l.amount,l.logTimestamp,l.additionalParams,l.groupKey, `a`.`pluginKey`, `a`.`actionKey` FROM `'.$this->getTableName().'` AS `l`
            INNER JOIN `'.$actionDao->getTableName().'` AS `a` ON (`a`.`id` = `l`.`actionId`)
            WHERE `l`.`userId` = :uid AND ( groupKey IS NULL OR groupKey = \'\' )

            UNION

            SELECT l.id,l.userId,l.actionId,SUM(l.amount) as `amount`, MAX(l.logTimestamp) as `logTimestamp`,l.additionalParams,l.groupKey, `a`.`pluginKey`, `a`.`actionKey` FROM `'.$this->getTableName().'` AS `l`
            INNER JOIN `'.$actionDao->getTableName().'` AS `a` ON (`a`.`id` = `l`.`actionId`)
            WHERE `l`.`userId` = :uid AND groupKey IS NOT NULL AND groupKey != \'\'
            GROUP BY l.groupKey, `a`.`pluginKey`, `a`.`actionKey`, l.userId

            ORDER BY 5 DESC
            LIMIT :start, :limit';

        return $this->dbo->queryForList($sql, array('uid' => $userId, 'start' => $start, 'limit' => $limit));
    }

    /**
     * @param $userId
     * @return int
     */
    public function countEntriesForUser( $userId )
    {
        $example = new OW_Example();
        $example->andFieldEqual('userId', $userId);

        return $this->countByExample($example);
    }
    
    public function deleteUserCreditLogByUserId( $userId )
    {
        $example = new OW_Example();
        $example->andFieldEqual('userId', $userId);
        
        return $this->deleteByExample($example);
    }
}