<?php

/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class USERCREDITS_BOL_PackDao extends OW_BaseDao
{
    /**
     * Plugin key
     */
    CONST PLUGIN_KEY = 'usercredits';

    /**
     * Entity key
     */
    CONST ENTITY_KEY = 'user_credits_pack';

    /**
     * Singleton instance.
     *
     * @var USERCREDITS_BOL_PackDao
     */
    private static $classInstance;

    /**
     * Constructor.
     *
     */
    protected function __construct()
    {
        parent::__construct();
    }

    /**
     * Returns an instance of class.
     *
     * @return USERCREDITS_BOL_PackDao
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * @see OW_BaseDao::getDtoClassName()
     *
     */
    public function getDtoClassName()
    {
        return 'USERCREDITS_BOL_Pack';
    }

    /**
     * @see OW_BaseDao::getTableName()
     *
     */
    public function getTableName()
    {
        return OW_DB_PREFIX . 'usercredits_pack';
    }
    
    /**
     * Returns list of packs for account type
     *
     * @param $accountTypeId
     * @return array
     */
    public function getAllPacks( $accountTypeId = null )
    {
        $example = new OW_Example();
        if ( $accountTypeId )
        {
            $example->andFieldEqual('accountTypeId', $accountTypeId);
        }
        $example->setOrder('`price` ASC');
        
        return $this->findListByExample($example);
    }

    /**
     * Returns packs history count
     *
     * @return integer
     */
    public function findPacksHistoryCount()
    {
        $billingDao = BOL_BillingSaleDao::getInstance();

        $sql = '
            SELECT
                COUNT(*) AS `logsCount`
            FROM
                `' . $billingDao->getTableName() . '` AS `a`
            INNER JOIN
                `' . $this->getTableName() . '` AS `b`
            ON
                `b`.`id` = a.`entityId`
            WHERE
                `a`.`pluginKey` = :pluginKey
                    AND
                `a`.`entityKey` = :entityKey
                    AND
                `a`.`status` = :status';

        return $this->dbo->queryForColumn($sql, array(
            'pluginKey' => self::PLUGIN_KEY,
            'entityKey' => self::ENTITY_KEY,
            'status' => BOL_BillingSaleDao::STATUS_DELIVERED
        ));
    }

    /**
     * Returns packs history
     *
     * @param integer $page
     * @param integer $limit
     * @return array
     */
    public function findPacksHistory( $page, $limit )
    {
        $billingDao = BOL_BillingSaleDao::getInstance();

        $start = ($page - 1) * $limit;
        $sql = '
            SELECT
                  `a`.`id`,
                  `a`.`userId`,
                  `a`.`entityDescription`,
                  ROUND(`a`.`price`) AS `price`,
                  `a`.`currency`,
                  `a`.`timeStamp`
            FROM
                `' . $billingDao->getTableName() . '` AS `a`
            INNER JOIN
                `' . $this->getTableName() . '` AS `b`
            ON
                `b`.`id` = a.`entityId`
            WHERE
                `a`.`pluginKey` = :pluginKey
                    AND
                `a`.`entityKey` = :entityKey
                    AND
                `a`.`status` = :status
            ORDER BY
                `a`.`timeStamp` DESC
            LIMIT
                :start, :limit
        ';

        return $this->dbo->queryForList($sql, array(
            'pluginKey' => self::PLUGIN_KEY,
            'entityKey' => self::ENTITY_KEY,
            'status' => BOL_BillingSaleDao::STATUS_DELIVERED,
            'start' => $start,
            'limit' => $limit
        ));
    }
}