/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
OW_Winks = function( params )
{
    var listLoaded = false;

    var model = OW.Console.getData(params.key);
    var list = OW.Console.getItem(params.key);
    var counter = new OW_DataModel();

    counter.addObserver(this);

    this.onDataChange = function( data )
    {
        var newCount = data.get('new');
        var counterNumber = newCount > 0 ? newCount : data.get('all');

        list.setCounter(counterNumber, newCount > 0);

        if ( counterNumber > 0 )
        {
            list.showItem();
        }
    };

    list.onHide = function()
    {
        counter.set('new', 0);
        list.getItems().removeClass('ow_console_new_message');

        model.set('counter', counter.get());
    };

    list.onShow = function()
    {
        if ( !listLoaded )
        {
            this.loadList();
            listLoaded = true;
        }
    };

    model.addObserver(function()
    {
        if ( !list.opened )
        {
            counter.set(model.get('counter'));
        }
    });


    this.accept = function( key, userId, partnerId )
    {
        var item = list.getItem(key);
        var c = {};

        this.send('accept', userId, partnerId, function(data)
        {
            if ( !data || data.result === false )
            {
                OW.error(data.msg);
                
                return;
            }
            
            item.find('.ow_console_invt_toolbar').removeClass('ow_preloader');

            if ( data.onclick )
            {
                $(document.getElementById('send-message-' + key)).attr('onclick', data.onclick).show();
                OW.addScript(data.onclick);
            }
            else if ( data.url ) 
            {
                $(document.getElementById('send-message-' + key)).attr('href', data.url).show();
            }
        });
        
        $('.ow_console_invt_toolbar > a', item).hide();
        item.find('.ow_console_invt_toolbar').addClass('wink_btn_box ow_preloader');
        
        OW.trigger('winks.onChangeStatus', ['accept', userId, partnerId]);
        
        if ( item.hasClass('ow_console_new_message') )
        {
            c["new"] = counter.get("new") - 1;
        }
        
        c["all"] = counter.get("all") - 1;
        counter.set(c);

        return this;
    };

    this.ignore = function( key, userId, partnerId )
    {
        var item = list.getItem(key);
        var c = {};

        this.send('ignore', userId, partnerId, function(data)
        {
            if ( data && data.result === false )
            {
                OW.error(data.msg);
            }
        });
        
        OW.trigger('winks.onChangeStatus', ['ignore', userId, partnerId]);
        
        if ( item.hasClass('ow_console_new_message') )
        {
            c["new"] = counter.get("new") - 1;
        }
        
        c["all"] = counter.get("all") - 1;
        counter.set(c);

        list.removeItem(item);
        OW.info(OW.getLanguageText('winks', 'msg_ignore_request'));
        
        return this;
    };

    this.sendMessage = function( userId, partnerId, conversationId)
    {
        if ( userId === undefined || partnerId === undefined || conversationId === undefined )
        {
            return false;
        }
        
        OW.trigger('winks.onSendMessage', [userId, partnerId, conversationId]);
    };

    this.send = function( status, userId, partnerId, callback )
    {
        $.ajax({
            url: params.rsp,
            type: 'POST',
            cache: false,
            dataType: 'json',
            data:
            {
                funcName: 'changeStatus',
                status: status,
                userId: userId,
                partnerId: partnerId
            },
            success: callback
        });
    };
};
